!----------------------------------------------------------
!
!     Molecular Dynamics Simulation Package for Analysis of Solid Systems
!     (MD-SPASS)
!     Fortran90 Program for Classical Atomistic Simulations
!
!     Developed by Dr. Yoshi Umeno (umeno@iis.u-tokyo.ac.jp)
!     Laboratory of Computational Materials Science and Mechanics
!     Institute of Industrial Science
!     The University of Tokyo
!
!     Partly supported by Takahiro Shimada
!     Materials Science Laboratory
!     Dept. of Mechanical Engineering and Science
!     Graduate School of Engineering, Kyoto University
!
!----------------------------------------------------------

      program main
      use variables
      use constants
      use output
      use bookkeep_param
      use rp_param
      use debug_variables
      implicit double precision (a-h,o-z)

      write (6,*)
      write (6,*) '#####################################################'
      write (6,*) '##      Molecular Dynamics Simulation Package      ##'
      write (6,*) '##         for Analysis of Solid Systems           ##'
      write (6,*) '##                 (MD-SPASS)                      ##'
      write (6,*) '##     by   Yoshi Umeno (umeno@iis.u-tokyo.ac.jp)  ##'
      write (6,*) '##     and  Takahiro Shimada                       ##'
      write (6,*) '#####################################################'
      write (6,*)

!----------------------------------------------------
!----- SETTING --------------------------------------
!----------------------------------------------------
!---INPUT VALUE---
!---PBC
! RECOMMENDED: (T T F) for 2-D, (T F F) for 1-D
      pbcx = .true.
      pbcy = .true.
      pbcz = .true.

!---STRESS & TEMPERATURE CONTROL
      sgm_set      =  0.0d6     ! Stress (Pa) (Positive is tension)
      temp_set     =  100.0d0      ! Temperature (K)
      step_ite     =  100
      algorithm    =  0
        ! 0=Velocity Verlet, NVE (Allen & Tildesley)
        ! 1=Gear's P-C, NVE (Allen & Tildesley)
        ! 2=Gear, NVT (Nose thermostat)
        ! 3=Gear, NPH (Andersen pressure control)
        ! 4=Gear, NPT (Nose and Andersen)
        ! 5=Gear, NPH-PR (Parrinello-Rahman)
        ! 6=Gear, NPT-PR (Nose and Parrinello-Rahman)
        ! 7=Stress relaxation by annealing, no atom motion
        ! 8=Stress relaxation by annealing, atom motion by verlet
        ! 50=Gear, Stress relaxation by NPH-PR & damper
        ! 100=Configuration relaxation by NR
        ! 101=Atom relaxation by CG (Conjugate Gradient) method
        ! 102=Atom relaxation by FIRE (Bitzek et al. PRL97-170201)
        ! 103=Atom and cell relaxation by FIRE
        ! 104=Atom and cell relaxation by FIRE (test phase)
      vscale       =  0         ! Velocity scaling
      damper       =  0         ! Atom damping
      cellconstx   =  .false.   ! Cell constraint
      cellconsty   =  .false.   
      cellconstz   =  .false.   
      
!  Stress set
      sxx=0.0d6; syy=0.0d6; szz=0.0d6 ! <-- Edit here
      syz=0.0d6; szx=0.0d6; sxy=0.0d6 ! <-- Edit here
      sgmmat_set   = 0.0d0
      sgmmat_set(1,1) = sxx
      sgmmat_set(2,2) = syy
      sgmmat_set(3,3) = szz
      sgmmat_set(2,3) = syz !/2.0d0
      sgmmat_set(1,3) = szx !/2.0d0
      sgmmat_set(1,2) = sxy !/2.0d0
      sgmmat_set(3,2) = sgmmat_set(2,3)
      sgmmat_set(3,1) = sgmmat_set(1,3)
      sgmmat_set(2,1) = sgmmat_set(1,2)
      if (.not.pbcx) sgmmat_set(1,1)=0.0d0
      if (.not.pbcy) sgmmat_set(2,2)=0.0d0
      if (.not.pbcz) sgmmat_set(3,3)=0.0d0
      
!  Strain set
      exx=0.0d0; eyy=0.0d0; ezz=0.0d0 ! <-- Edit here
      eyz=0.0d0; ezx=0.0d0; exy=0.0d0 ! <-- Edit here
      strmat_set = 0.0d0
      strmat_set(1,1) = exx
      strmat_set(2,2) = eyy
      strmat_set(3,3) = ezz
      strmat_set(2,3) = eyz/2.0d0
      strmat_set(1,3) = ezx/2.0d0
      strmat_set(1,2) = exy/2.0d0
      strmat_set(3,2) = strmat_set(2,3)
      strmat_set(3,1) = strmat_set(1,3)
      strmat_set(2,1) = strmat_set(1,2)

!---SYSTEM SETTING
!      a=4.04959d-10
      a=3.61d-10
!      wm=am*13.0d0+bm*14.0d0
      call initconfsc(5,5,5)    ! Single crystal
      call strain               ! Apply strain matrix

!----- SETTING END ---------------------------------

!------------------------------------------------------
! Initialize variables
      call var_initialize

      call readcondition

      if (continue_md) then
         call readcont
      else
         call readconfig
!         if (ini_velocity) call initial_velocity
      endif
      hmat0 = hmat

! Set atom species
      call set_atom_species
! Weight of atom
      call atom_weight
      if (.not.continue_md) then
         if (ini_velocity) call initial_velocity
      endif

!-----
      step_start=step
      step_end=step+step_ite
      rc2=rc**2
      dt2=dt**2

!  Stress set
      sgmmat_set   = 0.0d0
      sgmmat_set(1,1) = sxx
      sgmmat_set(2,2) = syy
      sgmmat_set(3,3) = szz
      sgmmat_set(2,3) = syz !/2.0d0
      sgmmat_set(1,3) = szx !/2.0d0
      sgmmat_set(1,2) = sxy !/2.0d0
      sgmmat_set(3,2) = sgmmat_set(2,3)
      sgmmat_set(3,1) = sgmmat_set(1,3)
      sgmmat_set(2,1) = sgmmat_set(1,2)
      if (.not.pbcx) sgmmat_set(1,1)=0.0d0
      if (.not.pbcy) sgmmat_set(2,2)=0.0d0
      if (.not.pbcz) sgmmat_set(3,3)=0.0d0

      pex=-sgm_set
      pexmat=-sgmmat_set

! Weight of piston/heat-reservoir
! Recommended values:
! wwr=0.005d0   / wqr=1.5d0 for algo=2/3
! wwr=0.005d1   / wqr=1.5d1 for 4
! wwr=0.005d-35             for 5
! wwr=0.005d-33 / wqr=1.5d1 for 6
! wwr=0.005d-28             for 7,8
! wwr=0.005d-33             for 50
      if ((algorithm==2).or.(algorithm==3)) then
         wwr=0.005d0; wqr=1.5d0
      else if (algorithm==4) then
         wwr=0.005d1; wqr=1.5d1
      else if ((algorithm==5).or.(algorithm==50)) then
         wwr=0.005d-32
      else if (algorithm==6) then
         wwr=0.005d-33; wqr=1.5d1
      else if ((algorithm==7).or.(algorithm==8)) then
         wwr=0.005d-28
      else
         wwr=0.005d-33
      endif
      wwr=wwr*stress_weight
      wqr=wqr*temp_weight

      ww=wwr/a**4*wmave
      wq=wqr*wmave*a**2

! Set bookkeeping algorithm
      if (potential_func=='Tersoff') then
         bookkeep_algo = 2
      else if (potential_func=='TersoffNM') then
         bookkeep_algo = 2
      else if (potential_func=='StilWeb') then
         bookkeep_algo = 2
      else if (potential_func=='TersoffBelko') then
         bookkeep_algo = 2
      else if (potential_func=='MEAM') then
         bookkeep_algo = 2
      else
         bookkeep_algo = 1
      endif

! In the case of shell model
      if (potential_func=='Shellmodel') then
         allocate (tmprx(n/2,2),tmpry(n/2,2),tmprz(n/2,2)) 
         allocate (tmpfx(n/2,2),tmpfy(n/2,2),tmpfz(n/2,2)) 
         if ((.not.pbcx).or.(.not.pbcy).or.(.not.pbcz)) then
            print *,'For shell model calculation, only 3D periodic system is supported.'
            stop
         endif
      endif

!!! PHONON !!!
!      call phonon(1,1,1,0.0d0,0.0d0,0.0d0)
!      stop

! writing data
      call writedata

!===  1 step advance======================================

      loopflg = 0
      mdloop: do while (step < step_end)

! stress increase
      if ((dstress).and.(step>dstress_start)) then
         sgmmat_set(1,1) = sgmmat_set(1,1)+dsxx
         sgmmat_set(2,2) = sgmmat_set(2,2)+dsyy
         sgmmat_set(3,3) = sgmmat_set(3,3)+dszz
         sgmmat_set(2,3) = sgmmat_set(2,3)+dsyz !/2.0d0
         sgmmat_set(1,3) = sgmmat_set(1,3)+dszx !/2.0d0
         sgmmat_set(1,2) = sgmmat_set(1,2)+dsxy !/2.0d0
         sgmmat_set(3,2) = sgmmat_set(2,3)
         sgmmat_set(3,1) = sgmmat_set(1,3)
         sgmmat_set(2,1) = sgmmat_set(1,2)
         if (.not.pbcx) sgmmat_set(1,1)=0.0d0
         if (.not.pbcy) sgmmat_set(2,2)=0.0d0
         if (.not.pbcz) sgmmat_set(3,3)=0.0d0
         pex=-sgm_set
         pexmat=-sgmmat_set
      endif
         
      call mkq
      if ((mod(step,nbk)==0).or.(step==step_start)) then
         call out
         if (rpart) then
            call bookkeep_rp
         else
            call bookkeep
         endif
      endif
      step=step+1

!! Apply temperature when step=appl_temp_step (if set)
      if (step == appl_temp_step) then
         temp_set = appl_temp_temp1
         damper = -damper
         call initial_velocity
      else if ((step > appl_temp_step).and.(step < (appl_temp_step + appl_temp_length))) then
         call velocity_scaling
      else if (step == (appl_temp_step + appl_temp_length)) then
         temp_set = appl_temp_temp2
         call initial_velocity
         damper = -damper
      endif

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!  For debugging stress calculation
!      if ((step==1).or.(mod(step,50)==0)) then
!         call stresscheck
!      endif
!      stop
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!      call meam_RaveloSn_ref
!      call meam_HuangSiC_ref
!      stop
!
! For debugging force calculation
!
!      call debugpotential

! For debugging the force subroutine
! (useful for writing a new force routine)
!      call forcecheck
!      call forcecheck2
!      stop

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!  For debugging force calculation:
!!!   maybe helpful when developping a subroutine
!!!   for a new potential function
      if ((debug).and.(step==1)) then
         allocate (den1(n),den2(n))
         allocate (nn(n),nnnum(n,n))
         allocate (distance(n,n))
         allocate (fcom(n,n,3))
      endif
      if ((debug).and.(step==3)) then
         den1=0.0d0; den2=0.0d0
         nn=0; nnnum=0
         call forcecheck
         stop
      endif
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!-----Force calc & numerical integral-----
      qpx=qx;qpy=qy;qpz=qz
      call calccent(cntpx,cntpy,cntpz)
!      if (diffusion) then; rxp=rx; ryp=ry; rzp=rz; endif
      if (algorithm == 0) then
         call velocity_verlet_a
         call potential
         call velocity_verlet_b
         call kinetic
      else if (algorithm == 1) then
         call gear_pc_predic
         call potential
         call gear_pc_correc
         call kinetic
      else if (algorithm == 2) then
         call gear_pc_nvt_predic
         call potential
         call gear_pc_nvt_correc
      else if (algorithm == 3) then
         call gear_pc_nph_predic
         call potential
         call gear_pc_nph_correc
      else if (algorithm == 4) then
         call gear_pc_npt_predic
         call potential
         call gear_pc_npt_correc
      else if (algorithm == 5) then
         call gear_pc_nphpr_predic
         call potential
         call gear_pc_nphpr_correc
      else if (algorithm == 6) then
         call gear_pc_nptpr_predic
         call potential
         call gear_pc_nptpr_correc
      else if (algorithm == 7) then
         call potential
         call cellrlx
      else if (algorithm == 8) then
         call velocity_verlet_a
         call potential
         call velocity_verlet_b
         call kinetic
         call cellrlx
      else if (algorithm == 50) then
         call gear_pc_nphpr_predic_rlx
         call potential
         call gear_pc_nphpr_correc_rlx
!         vx=0.0d0;vy=0.0d0;vz=0.0d0
!         do i=1,3
!            do j=1,3
!               if (hamat(i,j)*hvmat(i,j)<0.0) then
!                  hvmat(i,j)=0.0d0
!               endif
!            enddo
!         enddo
!         if (step>2000) hvmat=0.0d0
      else if (algorithm == 100) then
         call potential
         call atom_relax
         call kinetic
      else if (algorithm == 101) then
         call atom_relax_cg
         call potential
      else
         print *,'Algorithm is not specified.'
         stop
      endif
      call calccent(cntx,cnty,cntz)

      if (fixdrift) then
         call fix_drift
      endif
      if (fixrotation) then
         call fix_rotation
      endif

! Total drift check
!      print *,'Total drift energy (eV)',real(totaldrift()/ev)

      if (vscale /= 0) then
         if (mod(step,vscale) == 0) then
            call velocity_scaling
         endif
      endif

! Atom motion constraint
      call fix_atom

! Damper for atom motion. 1=GLOC, 2=FIRE
      if (damper == 1) then
         call atom_damping
      else if (damper == 2) then
         call atom_relax_fire
      endif

! Diffusion
      if (diffusion) call calcmsd

! Kinetic energy
      call kinetic

      if ((cellconstx).or.(cellconsty).or.(cellconstz)) then
         call cellconstraint
      endif

!---output---
      pres=(enkin*2.0d0+virx+viry+virz)/v/3.0d0
      sgm=-pres
      call calcstresstensor
      temp=enkin*2.0d0/3.0d0/dble(n)/bl
      call writedata
!--- MD end criterion
      if ((tolene>0.0).or.(step>=step_end-2)) then
         ene0=ene1
         ene1=enpot/ev/n
         if (step-step_start>1) then
            if (abs(ene1-ene0)<tolene) loopflg=1
         endif
      endif
      if ((tolst>0.0).or.(step>=step_end-2)) then
         st0=st1
         st1=max(abs(maxval(sgmmat)),abs(minval(sgmmat)))
         if (step-step_start>1) then
            if (abs(st1-st0)<tolst) loopflg=3
         endif
      endif
      if ((tolfor>0.0).or.(step==step_end)) then
         force1=-1.0
         do i=1,n
            fvec=fx(i)**2+fy(i)**2+fz(i)**2
            if (force1<fvec) then
               force1=fvec
               iii=i
            endif
         enddo
         force1=dsqrt(force1)/ev*ang
         if (force1<tolfor) loopflg=2
         force0=force1
      endif
      if (loopflg>0) exit
!---  loop---
      enddo mdloop

      if (loopflg==0) then
         print *,'MD ended: maximum step number is reached'
      else if (loopflg==1) then
         print *,'MD ended: energy is converged'
      else if (loopflg==2) then
         print *,'MD ended: force is converged'
      else if (loopflg==3) then
         print *,'MD ended: stress is converged'
      endif
      print *,'Energy difference:',abs(ene1-ene0),' eV per atom'
      print *,'Maximum force:',force1,'eV/ang'
      call writedata

! Elastic constants (numerical evaluation): use with care
      if (econstd>0.0) then
         call calcelastic
      endif

! NEB
      if (neb) then
         call nebanalysis
      endif

! Instability check
      if (inst) then
         call mkq
         call out
         call bookkeep
         call instability_cellonly
!         call instability_atomonly;stop
         if (inst_decom) then
            call instability_atomcell_decompose
         else
            call instability_atomcell
         endif
      endif

! Voronoi polyhedron analysis
      if (rc_vor>0.0) then
         call voronoi
      endif

      print *
      print *,'## MD-SPASS END ##'
      print *,'Total warning(s):',iwarn
      atime=etime(btime)
      print *,'Total calculation time: ',btime,'  sec'
      print *

      end
